library("ggplot2")
library("reshape2")

kbcov_method_list <- factor(c("total.multivariance {multivariance}", 
                              "bcov.test {Ball}, Constant", 
                              "bcov.test {Ball}, Probability", 
                              "bcov.test {Ball}, Chi-Square"), ordered = TRUE,
                            levels = c("bcov.test {Ball}, Constant", 
                                       "bcov.test {Ball}, Probability", 
                                       "bcov.test {Ball}, Chi-Square",
                                       "total.multivariance {multivariance}"))

bcov_method_list <- factor(c("dcov.test {energy}", 
                             "hhg.test {HHG}", 
                             "bcov.test {Ball}, Constant", 
                             "bcov.test {Ball}, Probability", 
                             "bcov.test {Ball}, Chi-Square"), ordered = TRUE,
                           levels = c("bcov.test {Ball}, Constant", 
                                      "bcov.test {Ball}, Probability", 
                                      "bcov.test {Ball}, Chi-Square",
                                      "dcov.test {energy}", 
                                      "hhg.test {HHG}"))

bd_method_list <- factor(c("eqdist.test {energy}", 
                           "hhg.k.sample.test {HHG}", 
                           "bd.test {Ball}, Sum", 
                           "bd.test {Ball}, Max", 
                           "bd.test {Ball}, Sum-Max"), ordered = TRUE,
                         c("bd.test {Ball}, Sum", 
                           "bd.test {Ball}, Max", 
                           "bd.test {Ball}, Sum-Max",
                           "eqdist.test {energy}", 
                           "hhg.k.sample.test {HHG}"))

kbcov_method <- function(nrep) {
  m_list <- as.character(rep(c("total.multivariance {multivariance}", 
                               "bcov.test {Ball}, Constant", 
                               "bcov.test {Ball}, Probability", 
                               "bcov.test {Ball}, Chi-Square"
  ), nrep))
  m_list <- factor(m_list, ordered = TRUE, levels = c("bcov.test {Ball}, Constant", 
                                                      "bcov.test {Ball}, Probability", 
                                                      "bcov.test {Ball}, Chi-Square",
                                                      "total.multivariance {multivariance}"))
}

bcov_method <- function(nrep) {
  m_list <- as.character(rep(c("dcov.test {energy}", 
                               "hhg.test {HHG}", 
                               "bcov.test {Ball}, Constant", 
                               "bcov.test {Ball}, Probability", 
                               "bcov.test {Ball}, Chi-Square"
  ), nrep))
  m_list <- factor(m_list, ordered = TRUE, levels = c("bcov.test {Ball}, Constant", 
                                                      "bcov.test {Ball}, Probability", 
                                                      "bcov.test {Ball}, Chi-Square",
                                                      "dcov.test {energy}", 
                                                      "hhg.test {HHG}"))
}

kbd_method <- function(nrep) {
  m_list <- as.character(rep(c("eqdist.test {energy}", 
                               "hhg.k.sample.test {HHG}", 
                               "bd.test {Ball}, Sum", 
                               "bd.test {Ball}, Max", 
                               "bd.test {Ball}, Sum-Max"), nrep))
  m_list <- factor(m_list, ordered = TRUE, levels = c("bd.test {Ball}, Sum", 
                                                      "bd.test {Ball}, Max", 
                                                      "bd.test {Ball}, Sum-Max",
                                                      "eqdist.test {energy}", 
                                                      "hhg.k.sample.test {HHG}"))
}

example_list <- function(start_example, end_example, prefix = "4.1.", rep_num = 5) {
  prefix <- paste("Model", prefix)
  e_list <- as.character(rep(paste0(prefix, start_example:end_example), each = rep_num))
  e_list <- factor(e_list, ordered = TRUE, levels = paste0(prefix, start_example:end_example))
  e_list
}


bcov_plot_column_list <- c("40", "60", "80", "100", "120", "example", "Method")

kbcov_plot_column_list <- bcov_plot_column_list

bd_plot_column_list <- c("10", "20", "30", "40", "50", "example", "Method")

type1error_intercept <- geom_hline(yintercept = 0.05, linetype = "dashed")

basic_plot <- function(dat) {
  ggplot(dat, mapping = aes(x = variable, y = value, color = Method)) +
    facet_wrap( ~ example, ncol = 2) +
    geom_point() + geom_line() +
    scale_colour_brewer(type = "qual", palette = "Set1") +
    theme_bw() + 
    theme(legend.position = "bottom",
          text = element_text(color = "black"),
          axis.title = element_text(face = "bold", size = rel(1.3)),
          axis.text = element_text(size = rel(1), color = "black"),
          legend.title = element_text(face = "bold"),
          legend.text = element_text(face = "bold"),
          legend.background = element_rect(fill = "transparent"),
          legend.key.size = unit(0.8, 'lines')
    ) +
    guides(color = guide_legend(nrow = 2, byrow = TRUE))
}


data_process <- function(p_res) {
  p_res <- melt(p_res, c("example", "Method"))
  p_res[["variable"]] <- as.numeric(as.character(p_res[["variable"]]))
  p_res[["value"]] <- as.numeric(as.character(p_res[["value"]]))
  return(p_res)
}